==> audiochain-1.1.5-src.tgz: org/audiochain/io/BlockAudioDataReader.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import static java.lang.System.*;
import static java.util.Arrays.*;

import java.io.IOException;

public abstract class BlockAudioDataReader implements AudioDataReader, Omitable {

	protected AudioDataReader reader;
	private int blockSizeInFrames;
	private int remainderLength;
	private int[] remainderData;
	private int blockSize;
	protected boolean omit;
	
	public BlockAudioDataReader(AudioDataReader reader) {
		this.reader = reader;
	}

	@Override
	public int read(int[] data) throws IOException {
		int readLength = reader.read(data);

		if (omit || blockSizeInFrames <= 0) {
			return readLength;
		}

		int[] allData;
		int allDataLength;
		int block = blockSize;
		
		if(readLength==-1) {
			if(remainderData == null) {
				// the final read call, all remainder data has been processed
				return -1;
			} else {
				// process remainder
				for (int i = 0; i < remainderLength; i += block) {
					int len = remainderLength - i;
					if (len < block) {
						// write full, zero padded blocks if necessary
						processBlock(copyOfRange(remainderData, i, i + block), i, block);
					} else {
						processBlock(remainderData, i, block);
					}
				}
				remainderData = null;
				return -1;
			}
		} else {
			if(remainderData==null) {
				// no remainder yet (usually at first read calls)
				allData = data;
				allDataLength = readLength;
			} else {
				// merge new data with remainder data of last calls which were not processed yet
				allDataLength = readLength + remainderLength;
				allData = copyOf(remainderData, allDataLength);
				arraycopy(data, 0, allData, remainderLength, readLength);
			}
		}
		
		int processedLength = 0;
		if (allDataLength >= block) {
			for (int i = 0, n = allDataLength - block; i <= n && block > 0; i += block) {
				processBlock(allData, i, block);
				processedLength += block;
			}
		}

		remainderLength = allDataLength - processedLength;
		if (remainderLength > 0) {
			remainderData = copyOfRange(allData, processedLength, allDataLength);
		}

		return readLength;
	}

	abstract protected void processBlock(int[] block, int offset, int length);
	
	public int getBlockSizeInFrames() {
		return blockSizeInFrames;
	}

	public void setBlockSizeInFrames(int blockSizeInFrames) {
		this.blockSizeInFrames = blockSizeInFrames;
		this.blockSize = blockSizeInFrames << 1;
	}

	public int getBlockSize() {
		return blockSize;
	}

	public AudioDataReader getReader() {
		return reader;
	}

	public void setReader(AudioDataReader reader) {
		this.reader = reader;
	}

	@Override
	public void omit(boolean omit) {
		this.omit = omit;		
	}
	
	@Override
	public void seek(long framePosition) throws IOException {
		reader.seek(framePosition);
	}
	
	@Override
	public void stop() {
		reader.stop();
	}
	
	@Override
	public void close() throws IOException {
		reader.close();
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/io/BlockAudioDataReader.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import static java.lang.System.arraycopy;
import static java.util.Arrays.copyOf;
import static java.util.Arrays.copyOfRange;

import java.io.IOException;

public abstract class BlockAudioDataReader implements AudioDataReader, Omitable {

	protected AudioDataReader reader;
	private int blockSizeInFrames;
	private int remainderLength;
	private int[] remainderData;
	private int blockSize;
	protected boolean omit;
	
	public BlockAudioDataReader(AudioDataReader reader) {
		this.reader = reader;
	}

	@Override
	public int read(int[] data) throws IOException {
		int readLength = reader.read(data);

		if (omit || blockSizeInFrames <= 0) {
			return readLength;
		}

		int[] allData;
		int allDataLength;
		int block = blockSize;
		
		if(readLength==-1) {
			if(remainderData == null) {
				// the final read call, all remainder data has been processed
				return -1;
			} else {
				// process remainder
				for (int i = 0; i < remainderLength; i += block) {
					int len = remainderLength - i;
					if (len < block) {
						// write full, zero padded blocks if necessary
						processBlock(copyOfRange(remainderData, i, i + block), i, block);
					} else {
						processBlock(remainderData, i, block);
					}
				}
				remainderData = null;
				return -1;
			}
		} else {
			if(remainderData==null) {
				// no remainder yet (usually at first read calls)
				allData = data;
				allDataLength = readLength;
			} else {
				// merge new data with remainder data of last calls which were not processed yet
				allDataLength = readLength + remainderLength;
				allData = copyOf(remainderData, allDataLength);
				arraycopy(data, 0, allData, remainderLength, readLength);
			}
		}
		
		int processedLength = 0;
		if (allDataLength >= block) {
			for (int i = 0, n = allDataLength - block; i <= n && block > 0; i += block) {
				processBlock(allData, i, block);
				processedLength += block;
			}
		}

		remainderLength = allDataLength - processedLength;
		if (remainderLength > 0) {
			remainderData = copyOfRange(allData, processedLength, allDataLength);
		}

		return readLength;
	}

	abstract protected void processBlock(int[] block, int offset, int length);
	
	public int getBlockSizeInFrames() {
		return blockSizeInFrames;
	}

	public void setBlockSizeInFrames(int blockSizeInFrames) {
		this.blockSizeInFrames = blockSizeInFrames;
		this.blockSize = blockSizeInFrames << 1;
	}

	public int getBlockSize() {
		return blockSize;
	}

	public AudioDataReader getReader() {
		return reader;
	}

	public void setReader(AudioDataReader reader) {
		this.reader = reader;
	}

	@Override
	public void omit(boolean omit) {
		this.omit = omit;		
	}
	
	@Override
	public void seek(long framePosition) throws IOException {
		reader.seek(framePosition);
	}
	
	@Override
	public void stop() {
		reader.stop();
	}
	
	@Override
	public void close() throws IOException {
		reader.close();
	}
}
